#include "TArtCalibPPAC.h" 
#include "TArtPPAC.h"
#include "TArtPPACPara.h"
#include "TArtBigRIPSParameters.h"
#include "TArtRawEventObject.h"
#include "TArtTOF.h" 
#include "TArtCore.h" 
#include "TArtStoreManager.h" 

#include <TROOT.h>
#include <TMath.h>
#include <TClonesArray.h>

//__________________________________________________________
TArtCalibPPAC::TArtCalibPPAC()
  :fReconstructed(false)
{
  TArtCore::Info(__FILE__,"Creating the PPAC detector objects...");
  sman = TArtStoreManager::Instance();
  setup = (TArtBigRIPSParameters*)sman->FindParameters("BigRIPSParameters");
  if(!setup){
    TArtCore::Info(__FILE__,"Can not find parameter handler: BigRIPSParameters");
    throw;
  }

  fPPACArray = new TClonesArray("TArtPPAC",10);
  fPPACArray->SetName("BigRIPSPPAC");
  fPPACArray->SetOwner();
  sman->AddDataContainer(fPPACArray);
}

//__________________________________________________________
TArtCalibPPAC::~TArtCalibPPAC()  {
  ClearData();
}

//__________________________________________________________
void TArtCalibPPAC::LoadData()   {

  TArtRawEventObject *fEvent = (TArtRawEventObject *)sman->FindDataContainer("RawEvent");

  ClearData();
  for(int i=0;i<fEvent->GetNumSeg();i++){
    TArtRawSegmentObject *seg = fEvent->GetSegment(i);
    int detector = seg->GetDetector();
    if(PPACT == detector || PPACQ == detector) LoadData(seg);
  }
  return;

}

//__________________________________________________________
void TArtCalibPPAC::LoadData(TArtRawSegmentObject *seg)   {

  int device   = seg->GetDevice();
  int fpl      = seg->GetFP();
  int detector = seg->GetDetector();
  if(!(PPACT == detector || PPACQ == detector)) return;

  for(int j=0;j<seg->GetNumData();j++){
    TArtRawDataObject *d = seg->GetData(j);
    int geo = d->GetGeo(); 
    int ch = d->GetCh(); 
    unsigned int val = d->GetVal();
    TArtRIDFMap mm(fpl,detector,geo,ch);
    TArtPPACPara *para = (TArtPPACPara*)setup->GetPPACPara(&mm);
    if(NULL == para){
      TArtCore::Info(__FILE__,"Could not find TArtPPACPara...: Dev:%d, %s", device, mm.GetMapInfo());
      continue;
    }
    else {
      TArtCore::Debug(__FILE__,"Find TArtPPACPara...: Dev:%d, %s, : %s", device, mm.GetMapInfo(), para->GetDetectorName()->Data());
    }

    int id = para->GetID();
    int nppac = fPPACArray->GetEntries();
    TArtCore::Debug(__FILE__,"Entries in PPACArray: %d, PPAC-ID:%d", nppac, id);
    TArtPPAC * ppac = FindPPAC(id);

    if(NULL==ppac){
      new ((*fPPACArray)[nppac]) TArtPPAC();
      ppac = (TArtPPAC *)fPPACArray->At(nppac);
      ppac->SetID(id);
      fPPACParaArray.push_back(para);
    }

    // set raw data
    if(PPACQ == detector){
      if(mm == *((TArtRIDFMap *)para->GetQX1Map()))
	ppac->SetQX1Raw(val);
      else if(mm == *((TArtRIDFMap *)para->GetQX2Map()))
	ppac->SetQX2Raw(val);
      else if(mm == *((TArtRIDFMap *)para->GetQY1Map()))
	ppac->SetQY1Raw(val);
      else if(mm == *((TArtRIDFMap *)para->GetQY2Map()))
	ppac->SetQY2Raw(val);
      else if(mm == *((TArtRIDFMap *)para->GetQAMap()))
	ppac->SetQARaw(val);
    }
    if(PPACT == detector){
      if(mm == *((TArtRIDFMap *)para->GetTX1Map()))
	ppac->SetTX1Raw(val);
      else if(mm == *((TArtRIDFMap *)para->GetTX2Map()))
	ppac->SetTX2Raw(val);
      else if(mm == *((TArtRIDFMap *)para->GetTY1Map()))
	ppac->SetTY1Raw(val);
      else if(mm == *((TArtRIDFMap *)para->GetTY2Map()))
	ppac->SetTY2Raw(val);
      else if(mm == *((TArtRIDFMap *)para->GetTAMap()))
	ppac->SetTARaw(val);
    }

  }

  fDataLoaded = true;
  return;
}

//__________________________________________________________
void TArtCalibPPAC::ClearData()   {
  fPPACArray->Clear();
  fPPACParaArray.clear();
  fDataLoaded = false;
  fReconstructed = false;
  return;
}

//__________________________________________________________
void TArtCalibPPAC::ReconstructData()   { // call after the raw data are loaded

  if(!fDataLoaded) LoadData();

  for(int i=0;i<GetNumPPAC();i++){

    TArtPPAC *ppac = (TArtPPAC*)fPPACArray->At(i);
    TArtPPACPara *para = fPPACParaArray[i];

    Double_t fX      = -999.;
    Double_t fY      = -999.;
    bool fFiredX = false;     
    bool fFiredY = false;
 
    Double_t  fTSumX = -9999.; 
    Double_t  fTSumY = -9999.; 
    Double_t  fTDiffX = -9999.;
    Double_t  fTDiffY = -9999.;

    Double_t fTX1Raw = (Double_t)ppac->GetTX1Raw();
    Double_t fTX2Raw = (Double_t)ppac->GetTX2Raw();
    Double_t fTY1Raw = (Double_t)ppac->GetTY1Raw();
    Double_t fTY2Raw = (Double_t)ppac->GetTY2Raw();
    Double_t fTARaw  = (Double_t)ppac->GetTARaw();

    Double_t fTX1 = fTX1Raw * para->GetCh2NsX1();
    Double_t fTX2 = fTX2Raw * para->GetCh2NsX2();
    Double_t fTY1 = fTY1Raw * para->GetCh2NsY1();
    Double_t fTY2 = fTY2Raw * para->GetCh2NsY2();
    Double_t fTA  = fTARaw  * para->GetCh2NsA();
   
    if(fTX1Raw >0 && fTX1Raw < 4000. && fTX2Raw >0 && fTX2Raw < 4000. ) {
      fTSumX = fTX1 + fTX2 - 2*fTA;
      fTDiffX = fTX1 - fTX2;
    }
    if(fTY1Raw >0 && fTY1Raw < 4000. && fTY2Raw >0 && fTY2Raw < 4000. ) {
      fTSumY = fTY1 + fTY2 - 2*fTA;
      fTDiffY = fTY1 - fTY2;
    }

    if((para->GetTXSumMin() >= para->GetTXSumMax()) || 
       (fTSumX >= para->GetTXSumMin() && fTSumX <= para->GetTXSumMax()) ){  
      fTDiffX = fTDiffX - para->GetXDTimeOffset();
      fX = fTDiffX * para->GetXFactor() / 2.;
      fX -= para->GetXOffset();
      fX -= para->GetXPosOffset();
      fX = -fX; // definition for optics only for X
      ppac->SetFiredX(true);
      //TArtCore::Info(__FILE__,"%s: RawTX1:%f, RawTX2:%f, TX1:%f, TX2:%f, TXdiff:%f, fX:%f,  Ch2NsX1:%f, Ch2NsX2:%f, XDTimeOffset:%f, XFactor:%f, XOffset:%f, XPosOffset:%f, X:%f", para->GetDetectorName()->Data(), fTX1Raw, fTX2Raw, fTX1, fTX2, fTDiffX, fX, para->GetCh2NsX1(), para->GetCh2NsX2(), para->GetXDTimeOffset(), para->GetXFactor(), para->GetXOffset(), para->GetXPosOffset(),fX);
    }
    if((para->GetTYSumMin() >= para->GetTYSumMax()) || 
       (fTSumY >= para->GetTYSumMin() && fTSumY <= para->GetTYSumMax()) ){  
      fTDiffY = fTDiffY - para->GetYDTimeOffset();
      fY = fTDiffY * para->GetYFactor() / 2.;
      fY -= para->GetYOffset();
      fY -= para->GetYPosOffset();
      ppac->SetFiredY(true);
    }

    ppac->SetX(fX);
    ppac->SetY(fY);

    // copy some information from para to data container
    ppac->SetID(para->GetID());
    ppac->SetDetectorName(*(para->GetDetectorName()));
    ppac->SetFpl(para->GetFpl());
    ppac->SetXZPos(para->GetXZPos());
    ppac->SetYZPos(para->GetYZPos());

  }

  fReconstructed = true;
  return;
}

//__________________________________________________________
TArtPPAC * TArtCalibPPAC::GetPPAC(int i) {
  return (TArtPPAC *)fPPACArray->At(i);
}
//__________________________________________________________
TArtPPACPara * TArtCalibPPAC::GetPPACPara(int i) {
  return (TArtPPACPara *)fPPACParaArray[i];
}
//__________________________________________________________
Int_t TArtCalibPPAC::GetNumPPAC() {
  return fPPACArray->GetEntries();
}
//__________________________________________________________
TArtPPAC * TArtCalibPPAC::FindPPAC(Int_t id){
  for(int i=0;i<GetNumPPAC();i++)
    if(id == ((TArtPPAC*)fPPACArray->At(i))->GetID())
      return (TArtPPAC*)fPPACArray->At(i);
  return NULL;
}
//__________________________________________________________
TArtPPAC * TArtCalibPPAC::FindPPAC(TString *n){
  for(int i=0;i<GetNumPPAC();i++)
    if(*n == *(((TArtPPAC*)fPPACArray->At(i))->GetDetectorName()))
      return (TArtPPAC*)fPPACArray->At(i);
  TArtCore::Info(__FILE__,"Fail to find TArtPPAC: %s", n->Data());
  return NULL;
}
//__________________________________________________________
TArtPPAC * TArtCalibPPAC::FindPPAC(char *n){
  for(int i=0;i<GetNumPPAC();i++)
    if(strcmp(((TArtPPAC*)fPPACArray->At(i))->GetDetectorName()->Data(),n) == 0)
      return (TArtPPAC*)fPPACArray->At(i);
  TArtCore::Info(__FILE__,"Fail to find TArtPPAC: %s", n);
  return NULL;
}
