#include "TArtCalibPlastic.h" 
#include "TArtPlastic.h"
#include "TArtPlasticPara.h"
#include "TArtBigRIPSParameters.h"
#include "TArtRawEventObject.h"
#include "TArtTOF.h" 
#include "TArtCore.h" 
#include "TArtStoreManager.h"

#include <TROOT.h>
#include <TMath.h>
#include <TClonesArray.h>

//__________________________________________________________
TArtCalibPlastic::TArtCalibPlastic()
  :fReconstructed(false), fDataLoaded(false)
{
  TArtCore::Info(__FILE__,"Creating the Plastic detector objects...");
  sman = TArtStoreManager::Instance();
  setup = (TArtBigRIPSParameters*)sman->FindParameters("BigRIPSParameters");
  if(!setup){
    TArtCore::Info(__FILE__,"Can not find parameter handler: BigRIPSParameters");
    throw;
  }

  fPlasticArray = new TClonesArray("TArtPlastic",10);
  fPlasticArray->SetName("BigRIPSPlastic");
  fPlasticArray->SetOwner();
  sman->AddDataContainer(fPlasticArray);
}

//__________________________________________________________
TArtCalibPlastic::~TArtCalibPlastic()  {
  ClearData();
}

//__________________________________________________________
void TArtCalibPlastic::LoadData()   {

  TArtRawEventObject *fEvent = (TArtRawEventObject *)sman->FindDataContainer("RawEvent");

  ClearData();
  for(int i=0;i<fEvent->GetNumSeg();i++){
    TArtRawSegmentObject *seg = fEvent->GetSegment(i);
    int detector = seg->GetDetector();
    if(PLAT == detector || PLAQ == detector) LoadData(seg);
  }
  return;

}

//__________________________________________________________
void TArtCalibPlastic::LoadData(TArtRawSegmentObject *seg)   {

  int device   = seg->GetDevice();
  int fpl      = seg->GetFP();
  int detector = seg->GetDetector();
  if(!(PLAT == detector || PLAQ == detector)) return;
  for(int j=0;j<seg->GetNumData();j++){
    TArtRawDataObject *d = seg->GetData(j);
    int geo = d->GetGeo(); 
    int ch = d->GetCh(); 
    unsigned int val = d->GetVal();
    TArtRIDFMap mm(fpl,detector,geo,ch);
    TArtPlasticPara *para = (TArtPlasticPara*)setup->GetPlasticPara(&mm);
    if(NULL == para){
      TArtCore::Info(__FILE__,"Could not find TArtPlasticPara...: Dev:%d, %s", device, mm.GetMapInfo());
      continue;
    }
    else {
      TArtCore::Debug(__FILE__,"Find TArtPlasticPara...: Dev:%d, %s, : %s", device, mm.GetMapInfo(), para->GetDetectorName()->Data());
    }

    int id = para->GetID();
    int npla = fPlasticArray->GetEntries();
    TArtCore::Debug(__FILE__,"Entries in PlasticArray: %d, Plastic-ID:%d", npla, id);
    TArtPlastic * pla = FindPlastic(id);

    if(NULL==pla){
      new ((*fPlasticArray)[npla]) TArtPlastic();
      pla = (TArtPlastic *)fPlasticArray->At(npla);
      pla->SetID(id);
      fPlasticParaArray.push_back(para);
    }

    // set raw data
    if(PLAQ == detector){
      if(mm==*((TArtRIDFMap *)para->GetQLMap()))
	pla->SetQLRaw(val);
      else if(mm==*((TArtRIDFMap *)para->GetQRMap()))
	pla->SetQRRaw(val);
    }
    if(PLAT == detector){
      if(mm==*((TArtRIDFMap *)para->GetTLMap()))
	pla->SetTLRaw(val);
      else if(mm==*((TArtRIDFMap *)para->GetTRMap()))
	pla->SetTRRaw(val);
    }

  }

  fDataLoaded = true;
  return;
}

//__________________________________________________________
void TArtCalibPlastic::ClearData()   {
  fPlasticArray->Clear();
  fPlasticParaArray.clear();
  fDataLoaded = false;
  fReconstructed = false;
  return;
}

//__________________________________________________________
void TArtCalibPlastic::ReconstructData()   { // call after the raw data are loaded

  if(!fDataLoaded) LoadData();

  for(int i=0;i<GetNumPlastic();i++){
    TArtPlastic *pla = (TArtPlastic*)fPlasticArray->At(i);
    TArtPlasticPara *para = fPlasticParaArray[i];

    Double_t fTimeL = -1000., fTimeR = -1000.;
    Bool_t fLFired = false, fRFired = false, fFired = false;
    Double_t fTLRaw = pla->GetTLRaw();
    Double_t fTRRaw = pla->GetTRRaw();
    Double_t fQLRaw = pla->GetQLRaw();
    Double_t fQRRaw = pla->GetQRRaw();

    Double_t fTime = -1; 
    Double_t fTimeLSlew = -1; 
    Double_t fTimeRSlew = -1; 
    Double_t fTimeSlew = -1; 

    if(fTLRaw>0 && fTLRaw<4000) {
      fLFired = true;
      fTimeL = fTLRaw * para->GetTCalLeft();
      if(fQLRaw>0){
	fTimeLSlew = fTLRaw + para->GetTLSlewA()/(TMath::Sqrt(fQLRaw)) + para->GetTLSlewB();
	fTimeLSlew = fTimeLSlew * para->GetTCalLeft();
      }
      else{
	fTimeLSlew = fTimeL;
      }
    }

    if(fTRRaw>0 && fTRRaw<4000) {
      fRFired = true;
      fTimeR = fTRRaw * para->GetTCalRight();
      if(fQRRaw>0){
	fTimeRSlew = fTRRaw + para->GetTRSlewA()/(TMath::Sqrt(fQRRaw)) + para->GetTRSlewB();
	fTimeRSlew = fTimeRSlew * para->GetTCalRight();
      }
      else{
	fTimeR = fTimeRSlew;
      }
    }
    if(fLFired && fRFired) fFired = true;

    if(fFired) {
      fTime = (fTimeL+fTimeR)/2.;
      fTimeSlew = (fTimeLSlew+fTimeRSlew)/2.;
      //TArtCore::Info(__FILE__,"ch %d: fpl: %d, tlraw: %d, trraw: %d, tl: %f, tr: %f, t:%f", i , fFocalPlane, fTLRaw,fTRRaw,(float)fTL,(float)fTR,(float)fTOF);
      //fTDiff = fTL-fTR;
      //fX     = fDT2X[0] * pow(fTDiff,2) + fDT2X[1] * fTDiff + fDT2X[2];
    }

    pla->SetTime(fTime);
    pla->SetTimeL(fTimeL);
    pla->SetTimeR(fTimeR);
    pla->SetTimeLSlew(fTimeLSlew);
    pla->SetTimeRSlew(fTimeRSlew);
    pla->SetTimeSlew(fTimeSlew);

    // copy some information from para to data container
    pla->SetID(para->GetID());
    pla->SetDetectorName(*(para->GetDetectorName()));
    pla->SetFpl(para->GetFpl());
    pla->SetZoffset(para->GetZoffset());

  }

  fReconstructed = true;
  return;
}

//__________________________________________________________
TArtPlastic * TArtCalibPlastic::GetPlastic(int i) {
  return (TArtPlastic *)fPlasticArray->At(i);
}
//__________________________________________________________
TArtPlasticPara * TArtCalibPlastic::GetPlasticPara(int i) {
  return (TArtPlasticPara *)fPlasticParaArray[i];
}
//__________________________________________________________
Int_t TArtCalibPlastic::GetNumPlastic() {
  return fPlasticArray->GetEntries();
}
//__________________________________________________________
TArtPlastic * TArtCalibPlastic::FindPlastic(Int_t id){
  for(int i=0;i<GetNumPlastic();i++)
    if(id == ((TArtPlastic*)fPlasticArray->At(i))->GetID())
      return (TArtPlastic*)fPlasticArray->At(i);
  return NULL;
}
//__________________________________________________________
TArtPlastic * TArtCalibPlastic::FindPlastic(TString *n){
  for(int i=0;i<GetNumPlastic();i++)
    if(*n == *(((TArtPlastic*)fPlasticArray->At(i))->GetDetectorName()))
      return (TArtPlastic*)fPlasticArray->At(i);
  TArtCore::Info(__FILE__,"Fail to find TArtPlastic: %s", n->Data());
  return NULL;
}
//__________________________________________________________
TArtPlastic * TArtCalibPlastic::FindPlastic(char *n){
  for(int i=0;i<GetNumPlastic();i++)
    if(strcmp(((TArtPlastic*)fPlasticArray->At(i))->GetDetectorName()->Data(),n) == 0)
      return (TArtPlastic*)fPlasticArray->At(i);
  TArtCore::Info(__FILE__,"Fail to find TArtPlastic: %s", n);
  return NULL;
}
