#include "TArtRecoBeam.h" 
#include "TArtBeam.h"
#include "TArtRIPS.h"
#include "TArtTOF.h"
#include "TArtIC.h"
#include "TArtBigRIPSParameters.h"
#include "TArtCore.h" 
#include "TArtStoreManager.h"

#include <TROOT.h>
#include <TClonesArray.h>

//__________________________________________________________
TArtRecoBeam::TArtRecoBeam()
  :fReconstructed(false)
{

  TArtCore::Info(__FILE__,"Creating the Beam objects...");
  sman = TArtStoreManager::Instance();

  fBeamArray = new TClonesArray("TArtBeam",10);
  fBeamArray->SetName("BigRIPSBeam");
  fBeamArray->SetOwner();
  sman->AddDataContainer(fBeamArray);

  clight = 299.792458; // mm/nsec
  mnucleon = 931.49432; // MeV

}

//__________________________________________________________
TArtRecoBeam::~TArtRecoBeam()  {
  ClearData();
}

//__________________________________________________________
TArtBeam * TArtRecoBeam::DefineNewBeam(TArtRIPS *rips, TArtTOF *tof, char * icname){

  char name[128];
  Int_t nbeam=GetNumBeam();
  new ((*fBeamArray)[nbeam]) TArtBeam();
  TArtBeam * beam = (TArtBeam *)fBeamArray->At(nbeam);

  sprintf(name,"Beam_rips%s_tof%s_ic%s",rips->GetDetectorName()->Data(),tof->GetDetectorName()->Data(),icname);
  TArtCore::Info(__FILE__,"define %s",name);
  beam->SetDetectorName(name);

  beam->SetNumRIPS(1);
  beam->SetRIPSName(*(rips->GetDetectorName()));
  beam->SetTOFName(*(tof->GetDetectorName()));
  beam->SetICName(icname);
  return beam;
}

//__________________________________________________________
TArtBeam * TArtRecoBeam::DefineNewBeam(TArtRIPS *urips, TArtRIPS *drips, TArtTOF *tof, char * icname){

  char name[128];
  Int_t nbeam=GetNumBeam();
  new ((*fBeamArray)[nbeam]) TArtBeam();
  TArtBeam * beam = (TArtBeam *)fBeamArray->At(nbeam);

  sprintf(name,"Beam_urips%s_drips%s_tof%s_ic%s",
	  urips->GetDetectorName()->Data(),drips->GetDetectorName()->Data(),
	  tof->GetDetectorName()->Data(),icname);
  TArtCore::Info(__FILE__,"define %s",name);
  beam->SetDetectorName(name);

  beam->SetNumRIPS(2);
  beam->SetUpstreamRIPSName(*(urips->GetDetectorName()));
  beam->SetDownstreamRIPSName(*(drips->GetDetectorName()));
  beam->SetTOFName(*(tof->GetDetectorName()));
  beam->SetICName(icname);
  return beam;
}

//__________________________________________________________
void TArtRecoBeam::ClearData()   {
  for(int i=0;i<GetNumBeam();i++)
    fBeamArray->At(i)->Clear();
  fReconstructed = false;
  return;
}

//__________________________________________________________
void TArtRecoBeam::ReconstructData()   { // call after the raw data are loaded

  TClonesArray * rips_array = (TClonesArray *)sman->FindDataContainer("BigRIPSRIPS");
  Int_t num_rips = rips_array->GetEntries();
  TClonesArray * tof_array = (TClonesArray *)sman->FindDataContainer("BigRIPSTOF");
  Int_t num_tof = tof_array->GetEntries();
  TClonesArray * ic_array = (TClonesArray *)sman->FindDataContainer("BigRIPSIC");
  Int_t num_ic = ic_array->GetEntries();

  for(int i=0;i<GetNumBeam();i++){

    TArtBeam * beam = GetBeam(i);
    TArtRIPS * upstream_rips = NULL;
    TArtRIPS * downstream_rips = NULL;
    TArtTOF * tof = NULL;
    TArtIC * ic = NULL;

    //////////////////////////////////////////////////////////////////
    // loading data
    bool FindAllObjects = true;

    for(int j=0;j<num_rips;j++){
      TArtRIPS *tmp_rips = (TArtRIPS *)rips_array->At(j);
      TString ripsname = *(tmp_rips->GetDetectorName());
      if(ripsname ==  *(beam->GetUpstreamRIPSName()))
	upstream_rips = tmp_rips; 
      if(ripsname ==  *(beam->GetDownstreamRIPSName()))
	downstream_rips = tmp_rips; 
    }

    if(NULL == upstream_rips){
      TArtCore::Info(__FILE__,"no upstream rips: %s",beam->GetUpstreamRIPSName()->Data());
      FindAllObjects = false;
    }
    if(NULL == downstream_rips){
      TArtCore::Info(__FILE__,"no downstream rips: %s",beam->GetDownstreamRIPSName()->Data());
      FindAllObjects = false;
    }

    for(int j=0;j<num_tof;j++){
      TArtTOF *tmp_tof = (TArtTOF *)tof_array->At(j);
      TString tofname = *(tmp_tof->GetDetectorName());
      if(tofname ==  *(beam->GetTOFName())){
	tof = tmp_tof; 
	break;
      }
    }
    if(NULL == tof){
      TArtCore::Info(__FILE__,"no tof: %s",beam->GetTOFName()->Data());
      FindAllObjects = false;
    }

    for(int j=0;j<num_ic;j++){
      TArtIC *tmp_ic = (TArtIC *)ic_array->At(j);
      TString icname = *(tmp_ic->GetDetectorName());
      if(icname ==  *(beam->GetICName())){
	ic = tmp_ic; 
	break;
      }
    }
    if(NULL == ic){
      TArtCore::Info(__FILE__,"no ic: %s",beam->GetICName()->Data());
      FindAllObjects = false;
    }

    if(!FindAllObjects) continue;
    //////////////////////////////////////////////////////////////////
    // start to reconstruct

    Double_t brho = 0;
    Double_t beta = 0;
    Double_t aoq = 0;
    Double_t zet = 0;

    Int_t fpl_ic = ic->GetFpl();

    if(1 == beam->GetNumRIPS()){
      brho = upstream_rips->GetBrho();
      beta = tof->GetBeta();
      Double_t gamma = tof->GetGamma();
      aoq = brho * clight / mnucleon / beta / gamma;
    }
    else if(beam->GetNumRIPS() == 2){
    Double_t brho1 = upstream_rips->GetBrho();
    Double_t brho2 = downstream_rips->GetBrho();
    Double_t ftime = tof->GetTOF();
    Double_t fl1 = upstream_rips->GetLength();
    Double_t fl2 = downstream_rips->GetLength();
    Double_t rbeta1;
    Double_t rbeta2;

    Double_t alpha  = brho2/brho1;
    Double_t a1     = TMath::Sqrt(alpha*alpha * clight*clight * ftime*ftime
			  + (TMath::Power(alpha,4) - alpha*alpha) * fl1*fl1
			  + (1 - alpha*alpha) * fl2*fl2);
  
    rbeta1 = ( a1*fl1 + fl2*clight*ftime ) / ( a1*clight*ftime + (1-alpha*alpha)*fl1*fl2 );
    Double_t gammaf = 1/sqrt(1-pow(rbeta1,2));
    rbeta2  = ( a1*fl1 + fl2*clight*ftime )/ ( clight*clight * ftime*ftime + (alpha*alpha-1)*fl1*fl1 );
    Double_t gammab = 1/sqrt(1-pow(rbeta2,2));
    aoq = brho1*clight/mnucleon/rbeta1/gammaf; // should be same as brho2/beta2/gamma2

    //upstream_rips->SetBeta(rbeta1);
    //downstream_rips->SetBeta(rbeta2);

    if( downstream_rips->GetUpstreamFpl() < fpl_ic )
      beta = rbeta2;
    else if( upstream_rips->GetDownstreamFpl() < fpl_ic )
      beta = rbeta1;
    else
      TArtCore::Info(__FILE__,"ic looks out of range!!: %d",fpl_ic);

    //TArtCore::Info(__FILE__,"brho1:%f, brho2:%f, aoq1: %f, aoq2: %f, org-beta:%f, beta1: %f, beta2:%f",brho1,brho2,aoq,brho2*clight/mnucleon/rbeta2/gammab,(fl1+fl2)/ftime/clight,rbeta1,rbeta2);

    }

    Double_t ionpair = ic->GetIonPair();
    Double_t de_v = TMath::Log(ionpair*beta*beta) - TMath::Log((1-beta*beta)) - beta*beta;
    zet = ic->GetZCoef(0)*TMath::Sqrt(ic->GetEnergySqSum()/de_v)*beta + ic->GetZCoef(1);

    beam->SetAoQ(aoq);
    beam->SetZet(zet);
    beam->SetBeta(beta);
  }

  fReconstructed = true;
  return;
}

//__________________________________________________________
TArtBeam * TArtRecoBeam::GetBeam(int i) {
  return (TArtBeam *)fBeamArray->At(i);
}
//__________________________________________________________
Int_t TArtRecoBeam::GetNumBeam() {
  return fBeamArray->GetEntries();
}
